package work.xuye.common.spel;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.expression.Expression;
import org.springframework.expression.spel.standard.SpelExpressionParser;
import org.springframework.stereotype.Component;
import org.springframework.util.DigestUtils;
import org.springframework.util.ObjectUtils;
import work.xuye.common.constant.CustomerDateTimeFormatter;

import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;

/**
 * @author xuye
 * @since 2023/3/8 12:36
 **/
@Slf4j
@Component
@RequiredArgsConstructor
public class CustomFunction {
    private final JsonPathHandler jsonPathHandler;

    private final SpelExpressionParser parser;

    public String jsonExtract(String json, String path) {
        String result = null;
        try {
            result = jsonPathHandler.extract(json, path);
        } catch (Exception e) {
            log.error("jsonExtra handler error, json: [{}], path: [{}]", json, path);
            throw new RuntimeException(e);
        }
        log.debug("jsonExtra handler, json: [{}], path: [{}], result: [{}]", json, path, result);
        return result;
    }

    public String jsonExtractReplaceAll(String json, String jsonPath, String regex, String replacement) {
        return this.jsonExtract(json, jsonPath).replaceAll(regex, replacement).trim();
    }

    public String jsonExtractAppend(String json, String path, String append) {
        String before = this.jsonExtract(json, path);
        String result = before + append;
        log.debug("append handler, before: [{}], after: [{}]", before, result);
        return result;
    }

    public String rfc1123(String json, String path) {
        String before = this.jsonExtract(json, path);
        String result = null;
        try {
            result = ZonedDateTime
                    .parse(before, DateTimeFormatter.RFC_1123_DATE_TIME)
                    .toLocalDateTime()
                    .format(CustomerDateTimeFormatter.READABLE_FORMATTER);
        } catch (Exception e) {
            log.error("rfc1123 handler error, before: [{}]", before);
            throw new RuntimeException(e);
        }
        log.debug("rfc1123 handler, before: [{}], after: [{}]", before, result);
        return result;
    }

    //https://deepq.yuque.com/deepq/bur3gk/hgcvin90uvgyaqmk#FUENY
    public String jnzTitle(String json, String titlePath, String descPath) {
        String result = this.jsonExtract(json, titlePath);
        if (!result.equals("快讯")) {
            log.debug("jnzTitle handler, title: [{}]", result);
            return result;
        } else {
            String desc = this.jsonExtract(json, descPath);
            String descTitle = desc.substring(desc.indexOf("【") + 1, desc.indexOf("】"));
            log.debug("jnzTitle handler, title from desc: [{}]", descTitle);
            return descTitle;
        }
    }

    //https://deepq.yuque.com/deepq/bur3gk/hgcvin90uvgyaqmk#FUENY
    public String jnzDesc(String json, String descPath) {
        String desc = this.jsonExtract(json, descPath);
        if (desc.startsWith("【") && desc.contains("】")) {
            desc = desc.substring(desc.indexOf("】") + 1);
            log.debug("jnzDesc handler, desc after remove: [{}]", desc);
            return desc;
        } else {
            log.debug("jnzDesc handler, desc: [{}]", desc);
            return desc;
        }
    }

    /**
     * 这个方法是为了兼容一些特殊的情况，比如：金牛座的快讯没有link，会导致无法入库，因此使用desc的md5值作为link
     * 优先使用primaryPath，如果primaryPath为空，则使用secondaryPathAsMd5的值
     *
     * @param json               json字符串
     * @param primaryPath        优先使用的路径
     * @param secondaryPathAsMd5 备选路径
     * @return 提取结果
     */
    public String orMd5(String json, String primaryPath, String secondaryPathAsMd5) {
        String primary = this.jsonExtract(json, primaryPath);
        if (!ObjectUtils.isEmpty(primary)) {
            log.debug("orMd5 handler, primary: [{}]", primary);
            return primary;
        } else {
            String sec = this.jsonExtract(json, secondaryPathAsMd5);
            String md5DigestAsHex = DigestUtils.md5DigestAsHex(sec.getBytes());
            log.debug("orMd5 handler, secondary: [{}], md5: [{}]", sec, md5DigestAsHex);
            return md5DigestAsHex;
        }
    }

    /**
     * !!需要注意使用了此方法的任务不要有更新逻辑！！
     * !!需要注意使用了此方法的任务不要有更新逻辑！！
     * !!需要注意使用了此方法的任务不要有更新逻辑！！
     * <p>
     * 作用是获得系统当前的时间，格式化为yyyy-MM-dd HH:mm:ss
     *
     * @return 当前时间
     */
    public String now() {
        Expression exp = parser.parseExpression("T(java.time.LocalDateTime).now().format(T(java.time.format.DateTimeFormatter).ofPattern('yyyy-MM-dd HH:mm:ss'))");
        return (String) exp.getValue();
    }

}
