package work.xuye.source.request;

import com.google.gson.Gson;
import lombok.extern.slf4j.Slf4j;
import org.springframework.expression.spel.standard.SpelExpressionParser;
import org.springframework.stereotype.Component;
import org.springframework.util.ObjectUtils;
import work.xuye.common.dto.HttpRequestParams;
import work.xuye.common.dto.HttpRes;
import work.xuye.common.enums.CharsetType;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.io.IOException;
import java.net.URI;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author xuye
 * @since 2023/2/18 00:22
 **/
@Slf4j
@Component
public class JdkRequestClient implements RequestClient {

    public static String regex = "#\\{.*?\\}";
    private final Pattern pattern = Pattern.compile(regex);
    @Resource
    private SpelExpressionParser parser;
    @Resource
    private Gson gson;
    private Map<CharsetType, Charset> charsetMap;

    @PostConstruct
    private void initCharsetMap() {
        if (charsetMap == null) {
            charsetMap = new HashMap<>();
        } else {
            charsetMap.clear();
        }
        charsetMap.put(CharsetType.defaultCharset, Charset.defaultCharset());
        charsetMap.put(CharsetType.utf8, StandardCharsets.UTF_8);
        charsetMap.put(CharsetType.gb2312, Charset.forName("GB2312"));
    }

    @Override
    public HttpRes execute(HttpRequestParams request) {
        HttpRequest httpRequest = null;
        this.preHandleUrl(request);
        this.preHandleBody(request);
        Map<String, Object> body = request.getBody();
        HttpRequest.BodyPublisher bodyPublisher;

        if (ObjectUtils.isEmpty(body)) {
            bodyPublisher = HttpRequest.BodyPublishers.noBody();
        } else {
            bodyPublisher = HttpRequest.BodyPublishers.ofString(gson.toJson(body));
        }


        //构建http请求：method，url，body，headers
        HttpRequest.Builder httpRequestBuilder = HttpRequest.newBuilder()
                .uri(URI.create(request.getUrl()))
                .method(request.getMethod(), bodyPublisher);

        request.getHeaders().forEach(httpRequestBuilder::header);
        httpRequest = httpRequestBuilder.build();


        HttpClient httpClient = HttpClient
                .newBuilder()
                .followRedirects(HttpClient.Redirect.ALWAYS)
                .build();

        HttpResponse<String> httpResponse = null;
        HttpResponse.BodyHandler<String> stringBodyHandler;
        if (request.getCharset() == null) {
            stringBodyHandler = HttpResponse.BodyHandlers.ofString();
        } else {
            Charset charset = charsetMap.getOrDefault(request.getCharset(),
                    Charset.defaultCharset());
            stringBodyHandler = HttpResponse.BodyHandlers.ofString(charset);
        }

        try {
            httpResponse = httpClient.send(httpRequest, stringBodyHandler);
        } catch (IOException | InterruptedException e) {
            throw new RuntimeException(e);
        }
//        if (httpResponse.statusCode() != 200) {
//            log.warn("http res code is not 200, code is {}", httpResponse.statusCode());
//            log.warn("http res body is {}", httpResponse.body());
//            log.warn("http res headers is {}", httpResponse.headers().map());
//            throw new RuntimeException("url is " + request.getUrl() + ", http code is not 200, code is "
//                    + httpResponse.statusCode());
//        }
        return HttpRes.build()
                .url(request.getUrl())
                .body(httpResponse.body())
                .status(httpResponse.statusCode())
                .headers(httpResponse.headers().map());
    }

    private void preHandleBody(HttpRequestParams request) {
        Map<String, Object> body = request.getBody();
        if (ObjectUtils.isEmpty(body)) {
            return;
        }
        for (String key : body.keySet()) {
            Object value = body.get(key);
            Matcher matcher = pattern.matcher(value.toString());
            if (matcher.find()) {
                String placeholder = matcher.group();
                String expr = request.getPlaceholderExpressions().get(placeholder);
                Object result = parser.parseExpression(expr).getValue();
                body.put(key, result);
            }
        }
        request.setBody(body);
    }

    private void preHandleUrl(HttpRequestParams request) {
        String url = request.getUrl();
        Matcher matcher = pattern.matcher(url);
        Map<String, Object> patternMap = new HashMap<>();
        while (matcher.find()) {
            String placeholder = matcher.group();
            patternMap.put(placeholder, null);
        }

        if (ObjectUtils.isEmpty(patternMap)) {
            return;
        }
        Map<String, String> placeholderExpressions = request.getPlaceholderExpressions();

        for (String key : patternMap.keySet()) {
            String expr = placeholderExpressions.get(key);
            Object value = parser.parseExpression(expr).getValue();
            patternMap.put(key, value);
        }
        for (String key : patternMap.keySet()) {
            url = url.replace(key, patternMap.get(key).toString());
        }
        request.setUrl(url);

    }
}
